/********************************************
  
  orbit7.c :  engine for space orbits

  by Mike Sussman, June 1998.

*******************************************/



/* 
 * Orbit7: A gravity simulator.
 * Copyright (C) 1998, 1999  Mike Sussman and others
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */



#include "orbit7.h"


/* Set Global variables for the Engine */


float time1=0;
float time_final=5;
float step=.01;
int num_elements = NUM_ELEMENTS;
float GM = 10*.04*.04;
float total_mass = 0;

/* Default values for physics constants */

double sun_mass = -5000;
double planet_mass = .02;
char spring_p = 0;

/*  Added by kfogel;  better randomness based on system time */

double my_random (double limit) {
  static time_t srand_seed = 0;
  double rdm;

  if (! srand_seed)
    srand (srand_seed = time (NULL));

  rdm = (double) rand();
  rdm /= ((double) RAND_MAX);
  rdm *= limit;
  return rdm;
}


/*   Initialize all objects  */

void generate_system() {

  /* mr. sun */
  position[0] = 0;
  position[1] = 0;
  position[2] = 0;
  position[3] = 0;
  position[4] = sun_mass;
  
  /* his little friends */
  {
    int counter = 1;
    while (counter < num_elements){
      double theta = my_random(6.283185307);
      double distance = 700 + 30*sqrt(my_random(200));
      position[5*counter] = distance * cos(theta);
      position[5*counter + 1] = distance * sin(theta);
      position[5*counter + 2] = sqrt(50000/distance) * sin(-theta);
      position[5*counter + 3] = sqrt(50000/distance) * cos(theta);
      position[5*counter + 4] = -planet_mass;
      counter++;
    }
  }
    
}


void init_total_mass() {

  int count1 = 0;
  total_mass = 0;
  while (count1 < num_elements){
    total_mass += position[5*count1 + 4];
    old_x[count1] = 0;
    old_y[count1] = 0;
    count1++;    
  }

}

/*  How to update the X coordinate  */

int x_func(int move_count){
  
  double zoof = win_width;
  double ziff = win_height;
  int new_x_index = 5*move_count;
  
  int old_x_index = 2*move_count;
  int old_y_index = 2*move_count + 1;
  int new_x_trajectory = 5*move_count + 2;
  float sum_x_trajectory = 0;
  int dip_count=0;
  
  while (dip_count< (num_elements)){
    
    r[dip_count] = sqrt(((old_position[old_x_index] - old_position[(old_x_index + (2*dip_count + 2)) % (2*num_elements)]) *
			 (old_position[old_x_index] - old_position[(old_x_index + (2*dip_count + 2)) % (2*num_elements)])) +
			((old_position[old_y_index] - old_position[(old_y_index + (2*dip_count + 2)) % (2*num_elements)]) *
			 (old_position[old_y_index] - old_position[(old_y_index + (2*dip_count + 2)) % (2*num_elements)])));
    
    if (r[dip_count] == 0)
      r[dip_count] = .00000001;
    dip_count++;
  }
  dip_count = 0;
  while (dip_count< (num_elements))
    {
      if(spring_p) {
	position[new_x_trajectory] += 
	  ((position[(5*move_count+(5*dip_count + 9)) % (5*num_elements)]) 
	   / total_mass) 
	  * (-(old_position[old_x_index] 
	       - old_position[(old_x_index + (2 * dip_count + 2)) 
			     % (2*num_elements)]) 
	     / r[dip_count])
	  * .000005 * (r[dip_count] * r[dip_count]);
      }
      else {
	position[new_x_trajectory] += 
	  ((position[(5*move_count+(5*dip_count + 9)) % (5*num_elements)]) 
	   / total_mass) 
	  * (-(old_position[old_x_index] 
	       - old_position[(old_x_index + (2 * dip_count + 2)) 
			     % (2*num_elements)]) 
	     / r[dip_count])
	  * 50000 / (r[dip_count] * r[dip_count]);
      }
      dip_count++;
    }
  position[new_x_index] += position[new_x_trajectory];
  {
    float x_scaled =  -(((ziff)/3000)*(position[new_x_index] - 120*time1 * sum_x_trajectory)) + ((zoof)/2);
    
    return (x_scaled);
  }
}


/*  How to update the Y coordinate */


int y_func(int move_count){

  float ziff = win_height;
  int new_y_index = 5*move_count + 1;
  int new_y_trajectory = 5*move_count + 3;
  float sum_y_trajectory = 0;
  int dip_count = 0;
  int old_y_index = 2*move_count + 1;
  
  dip_count = 0;
  while (dip_count< (num_elements)){
    if (spring_p) {
      position[new_y_trajectory]
	+= ((position[(5*move_count+(5*dip_count + 9)) % (5*num_elements)]) 
	    / total_mass) 
	* (-(old_position[old_y_index] 
	     - old_position[(old_y_index + (2 * dip_count + 2)) 
			   % (2*num_elements)])/r[dip_count]) 
	* .000005 * (r[dip_count] * r[dip_count]);
    }
    else {
      position[new_y_trajectory]
	+= ((position[(5*move_count+(5*dip_count + 9)) % (5*num_elements)]) 
	    / total_mass) 
	* (-(old_position[old_y_index] 
	     - old_position[(old_y_index + (2 * dip_count + 2)) 
			   % (2*num_elements)])/r[dip_count]) 
	* 50000 / (r[dip_count] * r[dip_count]);
    }
    dip_count++;
  }
  
  position[new_y_index] += position[new_y_trajectory];
 
  {
  float y_scaled = -(((ziff)/3000)*(position[new_y_index] - 120*time1*sum_y_trajectory)) + ((ziff)/2);
  
  return (y_scaled);
  }
}


/*   This is the function that is actually called repeatedly by the
//  X window event-loop.  In X toolkit parlance, a "work function".
//  The second-to-last line in start_X11_gui() registers this routine
//  as a work function.  
*/

int do_work()
{

  int move_count = 0;
  while(move_count < (num_elements)){
    int tmp_x = 2*move_count;
    int tmp_y = 2*move_count + 1;
    old_position[tmp_x] = position[5*move_count];
    old_position[tmp_y] = position[5*move_count + 1];
    old_trajectory[tmp_x] = position[5*move_count + 2];
    old_trajectory[tmp_y] = position[5*move_count + 3];
    move_count++;
  }
  move_count = 0;
  
  while (move_count < num_elements){
    int x_position = x_func(move_count);
    int y_position = y_func(move_count);
    int color;
    if (move_count < 1)
      color = 4;
    else
      color = 2;
    if (!((old_x[move_count] == x_position)
	  &&(old_y[move_count] == y_position))) {
      my_putpixel(x_position, y_position, color);
      my_putpixel(old_x[move_count], old_y[move_count], 0);
    }
    old_x[move_count] = x_position;
    
    old_y[move_count] = y_position;
    
    move_count++;      
  }
  time1 += step;

  return 1;
}


int main(int argc, char *argv[]) {
  generate_system();
  init_total_mass();
  start_X11_gui(argc, argv);
  return 0;
}


